@@ -76,6 +76,7 @@ char *mname;
 jmp_buf jabort;
 char *dotrans(), *domap();
 
+extern void intr();
 
 /*
  * `Another' gets another argument, and stores the new argc and argv.
@@ -89,7 +90,6 @@ another(pargc, pargv, prompt)
 	char *prompt;
 {
 	int len = strlen(line), ret;
-	extern void (*intr)();
 
 	if (len >= sizeof(line) - 3) {
 		printf("sorry, arguments too long\n");
@@ -164,6 +164,7 @@ setpeer(argc, argv)
 		}
 		port = htons(port);
 	}
+
 	host = hookup(argv[1], port);
 	if (host) {
 		int overbose;
