@@ -70,7 +70,7 @@ static char sccsid[] = "@(#)ftp.c	5.38 (Berkeley) 4/22/91";
 #include <pwd.h>
 #include <varargs.h>
 
-#if defined(SOLARIS2) || defined(__sgi)
+#if defined(SOLARIS2) || defined(__sgi) || defined(HPUX)
 #include <string.h>
 #include <unistd.h>
 #ifndef L_INCR
@@ -202,9 +202,11 @@ hookup(host, port)
 		goto bad;
 	}
 #ifdef IP_TOS
+#ifdef IPTOS_LOWDELAY
 	tos = IPTOS_LOWDELAY;
 	if (setsockopt(s, IPPROTO_IP, IP_TOS, (char *)&tos, sizeof(int)) < 0)
 		perror("ftp: setsockopt TOS (ignored)");
+#endif
 #endif
 	cin = fdopen(s, "r");
 	cout = fdopen(s, "w");
@@ -404,14 +406,23 @@ getreply(expecteof)
 				case WILL:
 				case WONT:
 					c = GETC(cin);
+#ifdef USE_SSL
 					FPRINTF(ssl_con,cout,
 					        "%c%c%c", IAC, DONT, c);
+#else /* !USE_SSL */
+					fprintf(cout,
+					        "%c%c%c", IAC, DONT, c);
+#endif /* !USE_SSL */
 					(void) FFLUSH(cout);
 					break;
 				case DO:
 				case DONT:
 					c = GETC(cin);
+#ifdef USE_SSL
 					FPRINTF(ssl_con,cout, "%c%c%c", IAC, WONT, c);
+#else /* !USE_SSL */
+					fprintf(cout, "%c%c%c", IAC, WONT, c);
+#endif /* !USE_SSL */
 					(void) FFLUSH(cout);
 					break;
 				default:
@@ -974,6 +985,16 @@ recvrequest(cmd, local, remote, lmode, printnames)
 				    (void) fflush(stdout);
 			    }
 		    }
+		    if ( c < -1 ) {
+			static char errbuf[1024];
+
+			sprintf(errbuf,"ftp: SSL_read DATA error %s\n",
+				    ERR_error_string(ERR_get_error()));
+
+			/* tell the user ... who else */
+			fprintf(stderr,"%s", errbuf);
+			fflush(stderr);
+		    }
 		} else 
 #endif /* !USE_SSL */
                 {
@@ -1197,9 +1218,11 @@ noport:
 	if (tmpno)
 		sendport = 1;
 #ifdef IP_TOS
+#ifdef IPTOS_THROUGHPUT
 	on = IPTOS_THROUGHPUT;
 	if (setsockopt(data, IPPROTO_IP, IP_TOS, (char *)&on, sizeof(int)) < 0)
 		perror("ftp: setsockopt TOS (ignored)");
+#endif
 #endif
 	return (0);
 bad:
@@ -1225,10 +1248,12 @@ dataconn(lmode)
 	(void) close(data);
 	data = s;
 #ifdef IP_TOS
+#ifdef IPTOS_THROUGHPUT
 	tos = IPTOS_THROUGHPUT;
 	if (setsockopt(s, IPPROTO_IP, IP_TOS, (char *)&tos, sizeof(int)) < 0)
 		perror("ftp: setsockopt TOS (ignored)");
 #endif
+#endif
 
 #ifdef USE_SSL
 	ssl_data_active_flag=0;
@@ -1238,7 +1263,7 @@ dataconn(lmode)
 	      SSL_free(ssl_data_con);
 	      ssl_data_con=NULL;
 	    }
-	    ssl_data_con=(SSL *)SSL_new();
+	    ssl_data_con=(SSL *)SSL_new(ssl_ctx);
 
 	    SSL_set_fd(ssl_data_con,data);
 	    SSL_set_verify(ssl_data_con,ssl_verify_flag,NULL);
@@ -1253,19 +1278,18 @@ dataconn(lmode)
 	    /* we are doing I/O and not using select so 
 	     * it is "safe" to read ahead
 	     */
-	    SSL_set_read_ahead(ssl_data_con,1);
+	    /* SSL_set_read_ahead(ssl_data_con,1); /**/
 
 	    if (debug) {
 		fprintf(stderr,"===>START SSL connect on DATA\n");
 		fflush(stderr);
 	    }
 
-	    if (!SSL_connect(ssl_data_con)) {
+	    if (SSL_connect(ssl_data_con)<=0) {
                 static char errbuf[1024];
 
-	        sprintf(errbuf,"ftp: SSL_connect DATA error %s %s\n",
-		            SSL_error_func_string(SSL_errno),
-			    SSL_error_string(SSL_errno));
+	        sprintf(errbuf,"ftp: SSL_connect DATA error %s\n",
+			    ERR_error_string(ERR_get_error()));
 
 		/* tell the user ... who else */
 		fprintf(stderr,"%s", errbuf);
@@ -1698,18 +1722,36 @@ char *u,*p;
 		return ERROR;
 	} else if (n == CONTINUE ) {
 	    /* do SSL */
-	    ssl_con=(SSL *)SSL_new();
+	    ssl_con=(SSL *)SSL_new(ssl_ctx);
 
 	    SSL_set_fd(ssl_con,fileno(cout));
 	    SSL_set_verify(ssl_con,ssl_verify_flag,NULL);
 	    SSL_set_pref_cipher(ssl_con,getenv("SSL_CIPHER"));
 
-	    if (!SSL_connect(ssl_con)) {
+	    /* Add in any certificates if you want to here ... */
+	    if (my_ssl_cert_file) {
+		if (!SSL_use_certificate_file(ssl_con, my_ssl_cert_file,
+		    X509_FILETYPE_PEM)) {
+		    fprintf(stderr,"%s: ",my_ssl_cert_file);
+		    ERR_print_errors(stderr);
+		    exit(1);
+		} else {
+		    if (!my_ssl_key_file)
+			my_ssl_key_file = my_ssl_cert_file;
+		    if (!SSL_use_RSAPrivateKey_file(ssl_con, my_ssl_key_file,
+			X509_FILETYPE_PEM)) {
+			fprintf(stderr,"%s: ", my_ssl_key_file);
+			ERR_print_errors(stderr);
+			exit(1);
+		    }
+		}
+	    }
+
+	    if (SSL_connect(ssl_con)<=0) {
                 static char errbuf[1024];
 
-	        sprintf(errbuf,"ftp: SSL_connect error %s %s\n",
-		            SSL_error_func_string(SSL_errno),
-			    SSL_error_string(SSL_errno));
+	        sprintf(errbuf,"ftp: SSL_connect error %s\n",
+			    ERR_error_string(ERR_get_error()));
 		perror(errbuf);
 		/* abort time methinks ... */
 		return ERROR;
