@@ -77,28 +77,32 @@ char	*getlogin();
 
 #ifdef USE_SSL
 
+/*
 #include "ssl_err.h"
+*/
 
 SSL *ssl_con;
+SSL_CTX *ssl_ctx;
 SSL *ssl_data_con;
 int ssl_debug_flag=0;
 int ssl_only_flag=0;
 int ssl_active_flag=0;
 int ssl_data_active_flag=0;
 int ssl_secure_flag=0;
+int ssl_certs_ok=0;
 int ssl_verify_flag=SSL_VERIFY_NONE;
 char ssl_file_path[1024];    /* don't look at that nasty value to the left */
 
 /* for the moment this is a compile time option only --tjh */
 int ssl_encrypt_data=1;
 
-static char *my_ssl_key_file=NULL;
-static char *my_ssl_cert_file=NULL;
+char *my_ssl_key_file=NULL;
+char *my_ssl_cert_file=NULL;
 
 #endif /* USE_SSL */
 
 #ifdef USE_SOCKS
-int socks_flag=0;
+int socks_flag=1;        /* socks usage is now the default */
 #endif /* USE_SOCKS */
 
 main(argc, argv)
@@ -109,12 +113,10 @@ main(argc, argv)
 	struct passwd *pw = NULL;
 	char homedir[MAXPATHLEN];
 	char *optarg;
+	char *argv0;
+
+        argv0=argv[0];
 
-	sp = getservbyname("ftp", "tcp");
-	if (sp == 0) {
-		fprintf(stderr, "ftp: ftp/tcp: unknown service\n");
-		exit(1);
-	}
 	doglob = 1;
 	interactive = 1;
 	autologin = 1;
@@ -159,6 +161,9 @@ main(argc, argv)
 				if (strcmp(optarg, "secure") == 0 ) {
 				    ssl_secure_flag=1;
 				}
+				if (strcmp(optarg, "certsok") == 0 ) {
+				    ssl_certs_ok=1;
+				}
 				if (strncmp(optarg, "verify=", strlen("verify=")) == 0 ) {
 				    ssl_verify_flag=atoi(optarg+strlen("verify="));
 				}
@@ -168,17 +173,54 @@ main(argc, argv)
 				if (strncmp(optarg, "key=", strlen("key=")) == 0 ) {
 				    my_ssl_key_file=optarg+strlen("key=");
 				}
-#ifdef USE_SOCKS
 				if (strcmp(optarg, "socks") == 0 ) {
+#ifdef USE_SOCKS
 				    socks_flag=1;
-				}
+#else /* !USE_SOCKS */
+                                    fprintf(stderr,"No SOCKS support\n");
+				    fflush(stderr);
 #endif /* USE_SOCKS */
+				}
+				if (strcmp(optarg, "!socks") == 0 ) {
+#ifdef USE_SOCKS
+				    socks_flag=0;
+#else /* !USE_SOCKS */
+                                    fprintf(stderr,"No SOCKS support\n");
+				    fflush(stderr);
+#endif /* USE_SOCKS */
+				}
                                 /* we have swallowed an extra arg */
 				argc--;
                                 argv++;
 				break;
 #endif /* USE_SSL */
 
+#ifndef USE_SSL
+                case 'z':
+                        if (strcmp(optarg, "socks") == 0 ) {
+#ifdef USE_SOCKS
+                            socks_flag=1;
+#else /* !USE_SOCKS */
+                            fprintf(stderr,"No SOCKS support\n");
+                            fflush(stderr);
+#endif /* USE_SOCKS */
+                        }
+                        if (strcmp(optarg, "!socks") == 0 ) {
+#ifdef USE_SOCKS
+                            socks_flag=0;
+#else /* !USE_SOCKS */
+                            fprintf(stderr,"No SOCKS support\n");
+                            fflush(stderr);
+#endif /* USE_SOCKS */
+                        }
+                        if (strcmp(optarg, "socks") == 0 ) {
+                            socks_flag=1;
+                        }
+                        break;
+#endif /* !USE_SSL */
+
+
+
 			default:
 				fprintf(stdout,
 				  "ftp: %c: unknown option\n", *cp);
@@ -189,9 +231,21 @@ main(argc, argv)
 
 #ifdef USE_SOCKS
         if (socks_flag) 
-	    SOCKSinit(argv[0]);
+	    SOCKSinit(argv0);
 #endif /* USE_SOCKS */
 
+        /* this was moved from before the arg processing to here
+	 * as if we build for socks then it wipes out the service
+	 * area with the getservbyname return for "socks" ... the
+	 * joys of a single static area and not copying the 
+	 * results of a lookup --tjh
+	 */
+	sp = getservbyname("ftp", "tcp");
+	if (sp == 0) {
+		fprintf(stderr, "ftp: ftp/tcp: unknown service\n");
+		exit(1);
+	}
+
 	fromatty = isatty(fileno(stdin));
 	if (fromatty)
 		verbose++;
@@ -232,10 +286,14 @@ main(argc, argv)
 		/* disable all the debug and trace */
 		SSL_LOG=SSL_ERR=NULL;
 	    }
+	    SSL_load_error_strings();
 
-            if (!X509_set_default_verify_paths()) {
+	    ssl_ctx=(SSL_CTX *)SSL_CTX_new();
+
+            if (!X509_set_default_verify_paths(ssl_ctx->cert)) {
 		fprintf(stderr,"ftp: cannot load certificates via X509_set_default_verify_paths\n");
 		fflush(stderr);
+		ERR_print_errors(stderr);
 		exit(1);
 	    }
 
