@@ -126,19 +126,30 @@ static char sccsid[] = "@(#)ftpd.c	5.40 (Berkeley) 7/2/91";
 
 #ifdef USE_SSL
 
+#include "rsa.h"        /* extra ... */
+/*
+#include "asn1.h"
+*/
+#include "x509.h"
+#include "pem.h"
 #include "ssl.h"
-#include "der.h"
-#include "X509.h"
-#include "PEM.h"
+/*
 #include "ssl_err.h"
+*/
 
 SSL *ssl_con;
+SSL_CTX *ssl_ctx;
 SSL *ssl_data_con;
 int ssl_debug_flag=0;
 int ssl_only_flag=0;
 int ssl_active_flag=0;
 int ssl_secure_flag=0;
 int ssl_verify_flag=SSL_VERIFY_NONE;
+int ssl_certsok_flag=0;
+int ssl_auto_login=0;
+
+static char *auth_ssl_name=NULL;
+
 
 int ssl_data_active_flag=0;
 
@@ -261,9 +272,11 @@ main(argc, argv, envp)
 		exit(1);
 	}
 #ifdef IP_TOS
+#ifdef IPTOS_LOWDELAY
 	tos = IPTOS_LOWDELAY;
 	if (setsockopt(0, IPPROTO_IP, IP_TOS, (char *)&tos, sizeof(int)) < 0)
 		syslog(LOG_WARNING, "setsockopt (IP_TOS): %m");
+#endif
 #endif
 	data_source.sin_port = htons(ntohs(ctrl_addr.sin_port) - 1);
 	debug = 0;
@@ -330,6 +343,9 @@ main(argc, argv, envp)
 		        if (strcmp(optarg, "secure") == 0 ) {
 			    ssl_secure_flag=1;
 			}
+                        if (strcmp(optarg, "certsok") == 0) {
+                            ssl_certsok_flag=1;
+                        }
 		        if (strncmp(optarg, "verify=", strlen("verify=")) == 0 ) {
 			    ssl_verify_flag=atoi(optarg+strlen("verify="));
 
@@ -372,7 +388,13 @@ nextopt:
             if (ssl_debug_flag) {
 		(void)freopen("ftpd.err","w",stderr);
 		SSL_ERR=stderr;
+		SSL_LOG=NULL;
                 SSL_debug("ftpd.log");
+		if (SSL_LOG==NULL) {
+		    SSL_LOG=fopen("ftpd.log","w");
+		    if (SSL_LOG==NULL)
+			SSL_LOG=stderr;
+		}
             } else {
                 /* disable all the debug and trace */
 		SSL_LOG=SSL_ERR=NULL; /**/
@@ -389,18 +411,28 @@ nextopt:
 		}
 	    }
 
+	    SSL_load_error_strings();
+
+	    ssl_ctx=SSL_CTX_new();
+
 	    /* I really should syslog any of the following
 	     * errors but I haven't bothered at this stage
 	     * as that can wait
 	     */
-            if (!X509_set_default_verify_paths()) {
+            if (!X509_set_default_verify_paths(ssl_ctx->cert)) {
 		fprintf(stderr,"ftpd: cannot set default path via X509_set_default_verify_paths\n");
 		fflush(stderr);
+		sleep(1);
 		exit(1);
             }
 
+#if 0
             sprintf(ssl_file_path,"%s/%s",X509_get_default_cert_dir(),
                         "ftpd.cert");
+#endif
+
+            sprintf(ssl_file_path,"%s/%s",X509_get_default_cert_dir(),
+                        "ftpd.pem");
 
 	    filename=my_ssl_cert_file==NULL?ssl_file_path:my_ssl_cert_file;
 	    fp=fopen(filename,"r");
@@ -411,17 +443,27 @@ nextopt:
 	    }
 
 	    ssl_public_cert=X509_new();
-
-	    if (PEM_read_X509(fp,ssl_public_cert)==0) {
+	    if (PEM_read_X509(fp,&ssl_public_cert,NULL)==NULL) {
 		fprintf(stderr,"ftpd: error reading public cert - %s\n",
-                                PEM_error_string(PEM_errno));
+                                ERR_error_string(ERR_get_error()));
 		fflush(stderr);
 		exit(1);
 	    }
 	    fclose(fp);
 
+	    if (ssl_debug_flag) {
+		fprintf(SSL_LOG,"ftpd: got public cert\n");
+		fflush(SSL_LOG);
+	    }
+
+#if 0
             sprintf(ssl_file_path,"%s/private/%s",X509_get_default_cert_area(),
                         "ftpd.key");
+#endif
+
+            sprintf(ssl_file_path,"%s/%s",X509_get_default_cert_dir(),
+                        "ftpd.pem");
+
 	    filename=my_ssl_key_file==NULL?ssl_file_path:my_ssl_key_file;
 	    fp=fopen(filename,"r");
 	    if (fp==NULL) {
@@ -431,14 +473,19 @@ nextopt:
 	    }
 
 	    ssl_private_key=RSA_new();
-	    if (PEM_read_RSA(fp,ssl_private_key)==0) {
+	    if (PEM_read_RSAPrivateKey(fp,&ssl_private_key,NULL)==0) {
 		fprintf(stderr,"ftpd: error reading private key - %s\n",
-                               PEM_error_string(PEM_errno));
+			ERR_error_string(ERR_get_error()));
 		fflush(stderr);
 		exit(1);
 	    }
 	    fclose(fp);
 
+	    if (ssl_debug_flag) {
+		fprintf(SSL_LOG,"ftpd: got private key\n");
+		fflush(SSL_LOG);
+	    }
+
         }
 #endif /* USE_SSL */
 
@@ -607,6 +654,12 @@ user(name)
 			return;
 		}
 	}
+#ifdef USE_SSL
+        if (pw && good_ssl_user(name)) {
+                reply(331, "Send dummy password to login.");
+                ssl_auto_login = 1;
+        } else
+#endif
 	reply(331, "Password required.");
 	askpasswd = 1;
 	/*
@@ -666,7 +719,11 @@ pass(passwd)
 		return;
 	}
 	askpasswd = 0;
-	if (!guest) {		/* "ftp" is only account allowed no password */
+	if (!guest
+#ifdef USE_SSL
+             && !ssl_auto_login
+#endif
+	   ) {		/* "ftp" is only account allowed no password */
 		if (pw == NULL)
 			salt = "xx";
 		else
@@ -926,9 +983,11 @@ getdatasock(mode)
 	}
 	(void) seteuid((uid_t)pw->pw_uid);
 #ifdef IP_TOS
+#ifdef IPTOS_THROUGHPUT
 	on = IPTOS_THROUGHPUT;
 	if (setsockopt(s, IPPROTO_IP, IP_TOS, (char *)&on, sizeof(int)) < 0)
 		syslog(LOG_WARNING, "setsockopt (IP_TOS): %m");
+#endif
 #endif
 	return (fdopen(s, mode));
 bad:
@@ -967,9 +1026,11 @@ dataconn(name, size, mode)
 		(void) close(pdata);
 		pdata = s;
 #ifdef IP_TOS
+#ifdef IPTOS_LOWDELAY
 		tos = IPTOS_LOWDELAY;
 		(void) setsockopt(s, IPPROTO_IP, IP_TOS, (char *)&tos,
 		    sizeof(int));
+#endif
 #endif
 		reply(150, "Opening %s mode data connection for %s%s.",
 		     type == TYPE_A ? "ASCII" : "BINARY", name, sizebuf);
@@ -1026,25 +1087,24 @@ dataconn(name, size, mode)
 	      SSL_free(ssl_data_con);
 	      ssl_data_con=NULL;
 	    }
-	    ssl_data_con=(SSL *)SSL_new();
+	    ssl_data_con=(SSL *)SSL_new(ssl_ctx);
 
 	    SSL_set_fd(ssl_data_con,data);
 	    SSL_set_verify(ssl_data_con,ssl_verify_flag,NULL);
 
 	    /* if is "safe" to read ahead */
-            SSL_set_read_ahead(ssl_data_con,1);
+            /* SSL_set_read_ahead(ssl_data_con,1); /**/
 
 	    if (ssl_debug_flag && (SSL_LOG!=NULL)) {
 		fprintf(SSL_LOG,"===>START SSL_accept on DATA\n");
 		fflush(SSL_LOG);
 	    }
 
-	    if (!SSL_accept(ssl_data_con)) {
+	    if (SSL_accept(ssl_data_con)<=0) {
                 static char errbuf[1024];
 
-	        sprintf(errbuf,"ftpd: SSL_accept DATA error %s %s\n",
-		            SSL_error_func_string(SSL_errno),
-			    SSL_error_string(SSL_errno));
+	        sprintf(errbuf,"ftpd: SSL_accept DATA error %s\n",
+			    ERR_error_string(ERR_get_error()));
 		perror_reply(425, errbuf);
 		/* abort time methinks ... */
 		fclose(file);
@@ -1131,13 +1191,12 @@ send_data(instr, outstr, blksize)
 			SSL_write(ssl_data_con, buf, cnt) == cnt)
 			    byte_count += cnt;
 		} else
-#else /* !USE_SSL */
+#endif /* USE_SSL */
                 {
 		while ((cnt = read(filefd, buf, (u_int)blksize)) > 0 &&
 		    write(netfd, buf, cnt) == cnt)
 			byte_count += cnt;
 		}
-#endif /* USE_SSL */
 		transflag = 0;
 		(void)free(buf);
 		if (cnt != 0) {
@@ -1195,7 +1254,7 @@ receive_data(instr, outstr)
 			    byte_count += cnt;
 		    }
 		} else 
-#else /* !USE_SSL */
+#endif /* !USE_SSL */
                 {
 		while ((cnt = read(fileno(instr), buf, sizeof buf)) > 0) {
 			if (write(fileno(outstr), buf, cnt) != cnt)
@@ -1203,7 +1262,6 @@ receive_data(instr, outstr)
 			byte_count += cnt;
 		}
 		}
-#endif /* !USE_SSL */
 		if (cnt < 0)
 			goto data_err;
 		transflag = 0;
@@ -1902,7 +1960,7 @@ char *fmt;
 #endif /* SETPROCTITLE */
 
 
-#ifdef __sgi
+#if defined(__sgi) || defined(DGUX)
 static FILE *fpShell;
 #define SHELL_FILE "/etc/shells"
 
@@ -1940,11 +1998,13 @@ int endusershell()
   }
 }
 
-#endif /* __sgi */
+#endif /* __sgi || DGUX */
 
 
 #ifdef USE_SSL
 
+static int verify_callback();
+
 do_ssl_start()
 {
     static char errstr[1024];
@@ -1955,27 +2015,28 @@ do_ssl_start()
     }
 
     /* do the SSL stuff now ... before we play with pty's */
-    ssl_con=(SSL *)SSL_new();
+    ssl_con=(SSL *)SSL_new(ssl_ctx);
 
     /* we are working with stdin (inetd based) by default */
     SSL_set_fd(ssl_con,0);
 
     if (SSL_use_RSAPrivateKey(ssl_con,ssl_private_key)==0) {
-        sprintf(errstr,"ftpd: SSL_use_RSAPrivateKey %s",SSL_error_string(SSL_errno));
+        sprintf(errstr,"ftpd: SSL_use_RSAPrivateKey %s",ERR_error_string(ERR_get_error()));
 	perror_reply(421, errstr);
 	dologout(1);
     }
 
     if (SSL_use_certificate(ssl_con,ssl_public_cert)==0) {
-        sprintf(errstr,"ftpd: SSL_use_certificate %s",SSL_error_string(SSL_errno));
+        sprintf(errstr,"ftpd: SSL_use_certificate %s",ERR_error_string(ERR_get_error()));
 	perror_reply(421, errstr);
 	dologout(1);
     }
 
-    SSL_set_verify(ssl_con,ssl_verify_flag,NULL);
+    SSL_set_verify(ssl_con,ssl_verify_flag,
+	    ssl_certsok_flag ? verify_callback : NULL);
 
-    if (!SSL_accept(ssl_con)) {
-	sprintf(errstr,"ftpd: SSL_accept %s %s",SSL_error_func_string(SSL_errno),SSL_error_string(SSL_errno));
+    if (SSL_accept(ssl_con)<=0) {
+	sprintf(errstr,"ftpd: SSL_accept %s %s",ERR_error_string(ERR_get_error()));
 
 	perror_reply(421, errstr);
 	dologout(1);
@@ -1996,10 +2057,13 @@ do_ssl_start()
 	ssl_active_flag=1;
 
 	if (logging)
-	    syslog(LOG_NOTICE, "SSL SUCCEEDED WITH %s", remotehost);
+            if (auth_ssl_name)
+                syslog(LOG_NOTICE, "SSL SUCCEEDED WITH %s as %s", remotehost,
+                    auth_ssl_name);
+            else
+                syslog(LOG_NOTICE, "SSL SUCCEEDED WITH %s", remotehost);
     }
 
-
     /* ssl_fprintf calls require that this be null to test
      * for being an ssl stream
      */
@@ -2086,5 +2150,81 @@ old_ssl_putc(SSL *ssl_con,int oneint)
     }
 }
 
+static int
+verify_callback(ok, xs, xi, depth, error)
+int ok;
+char *xs, *xi;
+int depth, error;
+{
+    /*
+     * If the verification fails, then don't remember the name.  However,
+     * if we don't require a certificate, then return success which will
+     * still allow us to set up an encrypted session.
+     *
+     */
+    if (!ok) {
+	/* If we can't verify the issuer, then don't accept the name. */
+    if (depth != 0 && auth_ssl_name) {
+		free(auth_ssl_name);
+ 		auth_ssl_name = 0;
+ 	}
+ 	return ssl_verify_flag & SSL_VERIFY_FAIL_IF_NO_PEER_CERT ? 0 : 1;
+     }
+     if (depth == 0)
+ 	auth_ssl_name =
+ 	    (char *)X509_NAME_oneline(X509_get_subject_name(xs));
+     return ok;
+}
+
+/* return true if this auth_ssl_name is authorized to use name. */
+int
+good_ssl_user(name)
+char *name;
+{
+   FILE *user_fp;
+   char buf[2048];
+
+   if (!auth_ssl_name)
+      return 0;
+   if (!ssl_certsok_flag)
+      return 0;	/* can't happen */
+   user_fp = fopen("/etc/ssl.users", "r");
+   if (!user_fp)
+      return 0;
+   while (fgets(buf, sizeof buf, user_fp)) {
+      char *cp;
+      char *n;
+
+      /* allow for comments in the file ... always nice
+       * to be able to add a little novel in files and
+       * also disable easily --tjh
+       */
+      if (buf[0]=='#')
+	  continue;
+
+      if (cp = strchr(buf, '\n'))
+	  *cp = '\0';
+      cp = strchr(buf, ':');
+      if (!cp)
+	  continue;
+      *cp++ = '\0';
+      if (strcasecmp(cp, auth_ssl_name) == 0) {
+	  n = buf;
+	  while (n) {
+	      cp = strchr(n, ',');
+	      if (cp)
+		  *cp++ = '\0';
+	      if (!strcmp(name, n)) {
+		  fclose(user_fp);
+		  return 1;
+	      }
+	      n = cp;
+	  }
+      }
+   }
+   fclose(user_fp);
+   return 0;
+}
+
 #endif /* USE_SSL */
 
